@@ -1,5 +1,4 @@
 //   -*- mode: c++; -*-
-
 /*
   
 Grammar of "string"'s:
@@ -26,6 +25,9 @@ Example "str":
    ifelse($longtitle, print(title), $longtitle)
 */
 
+#ifndef MENU_METHOD_H
+#define MENU_METHOD_H
+
 #include <iostream.h>
 #include <vector.h>
 #include <map.h>
@@ -39,13 +41,13 @@ class narg_mismatch:public parseerror{ //number of args mismatch to func.
 public:
   char name[MAX_LINE];
   narg_mismatch(parsestream *p,String s):parseerror(p){
-    strcpy(name,s.chrstr());
+    strcpy(name,s.c_str());
   };
   const char *errormsg(){
     String s=String("Number of arguments to function ")+
       String(name)+String(" doesn't match");
     static char err[MAX_LINE];
-    strcpy(err,s);
+    strcpy(err,s.c_str());
     return err;
   }
 };  
@@ -64,7 +66,7 @@ class dir_createerror{ //number of args mismatch to func.
 public:
   char name[MAX_LINE];
   dir_createerror(String s){
-    strcpy(name,s.chrstr());
+    strcpy(name,s.c_str());
   };
 };  
 
@@ -72,13 +74,13 @@ class missing_tag:public parseerror{
 public:
   char name[MAX_LINE];
   missing_tag(parsestream *p,String s):parseerror(p){
-    strcpy(name,s.chrstr());
+    strcpy(name,s.c_str());
   };
   const char *errormsg(){
     String s=String("Missing (or empty) tag: ")+
       name+"\n (This tag needs to defined for the menuentry to make sense)";
     static char err[MAX_LINE];
-    strcpy(err,s);
+    strcpy(err,s.c_str());
     return err;
   }
 };  
@@ -176,7 +178,7 @@ public:
 class func_def:public func{
   cat_str *f;
   String n;
-  vector <String> args_name;
+  StrVec args_name;
 public:
   func_def(parsestream &i);
   int nargs(){return args_name.size();};
@@ -334,6 +336,13 @@ public:
 		  map<String, String, less<String> > &);
   String name(){return "replacewith";}
 };
+class nstring_func:public func2{
+public: 
+  ostream &output(ostream &o, vector<cat_str *> &,
+		  map<String, String, less<String> > &);
+  String name(){return "nstring";}
+};
+
 class cppesc_func:public func1{
 public: 
   ostream &output(ostream &o, vector<cat_str *> &,
@@ -364,6 +373,81 @@ public:
 		  map<String, String, less<String> > &);
   String name(){return "entryindex";}
 };
+class firstentry_func:public func1{
+public: 
+  ostream &output(ostream &o, vector<cat_str *> &,
+		  map<String, String, less<String> > &);
+  String name(){return "firstentry";}
+};
+class lastentry_func:public func1{
+public: 
+  ostream &output(ostream &o, vector<cat_str *> &,
+		  map<String, String, less<String> > &);
+  String name(){return "lastentry";}
+};
+class level_func:public func0{
+public: 
+  ostream &output(ostream &o, vector<cat_str *> &,
+		  map<String, String, less<String> > &);
+  String name(){return "level";}
+};
+
+
+class rcfile_func:public func0{
+public: 
+  ostream &output(ostream &o, vector<cat_str *> &,
+		  map<String, String, less<String> > &);
+  String name(){return "rcfile";}
+};
+class examplercfile_func:public func0{
+public: 
+  ostream &output(ostream &o, vector<cat_str *> &,
+		  map<String, String, less<String> > &);
+  String name(){return "examplercfile";}
+};
+class mainmenutitle_func:public func0{
+public: 
+  ostream &output(ostream &o, vector<cat_str *> &,
+		  map<String, String, less<String> > &);
+  String name(){return "mainmenutitle";}
+};
+class rootsection_func:public func0{
+public: 
+  ostream &output(ostream &o, vector<cat_str *> &,
+		  map<String, String, less<String> > &);
+  String name(){return "rootsection";}
+};
+class rootprefix_func:public func0{
+public: 
+  ostream &output(ostream &o, vector<cat_str *> &,
+		  map<String, String, less<String> > &);
+  String name(){return "rootprefix";}
+};
+class userprefix_func:public func0{
+public: 
+  ostream &output(ostream &o, vector<cat_str *> &,
+		  map<String, String, less<String> > &);
+  String name(){return "userprefix";}
+};
+class treewalk_func:public func0{
+public: 
+  ostream &output(ostream &o, vector<cat_str *> &,
+		  map<String, String, less<String> > &);
+  String name(){return "treewalk";}
+};
+class postoutput_func:public func0{
+public: 
+  ostream &output(ostream &o, vector<cat_str *> &,
+		  map<String, String, less<String> > &);
+  String name(){return "postoutput";}
+};
+class preoutput_func:public func0{
+public: 
+  ostream &output(ostream &o, vector<cat_str *> &,
+		  map<String, String, less<String> > &);
+  String name(){return "preoutput";}
+};
+
 
 /////////////////////////////////////////////////////
 //  install-menu Function declarations
@@ -392,13 +476,37 @@ private:
 
   void check_config();
 public:
+  bool keep_sections;
   configinfo(parsestream &);
   cat_str *startmenu, *endmenu, *submenutitle, *hkexclude,
-          *genmenu, *postrun, *prerun;
+          *genmenu, *postrun, *prerun, *sort;
+
+  bool  onlyrunasroot, onlyrunasuser;
+  cat_str *preruntest;
+
+  // hint stuff:
+  bool   hint_optimize;
+  double hint_nentry;
+  double hint_topnentry;
+  double hint_mixedpenalty;
+  double hint_minhintfreq;
+  double hint_mlpenalty;
+  int    hint_max_ntry;
+  int    hint_max_iter_hint;
+  bool   hint_debug;
+
   int hotkeycase; //0=insensitive, 1=sensitive
   ostream &debuginfo(ostream &);
   
   String &compat(){return compt;};
+
+  //String &get_examplercfile(){return exrcf;};
+  //String &get_mainmenutitle(){return mainmt;};
+  //String &rootsection(){return roots;};
+  //String &userprefix(){return userpref;};
+  //String &treewalk(){return treew;};
+  //String &postoutput(){return postout;};
+  //String &preoutput(){return prerout;};
   String &rcfile(){return rcf;};
   String &examplercfile(){return exrcf;};
   String &mainmenutitle(){return mainmt;};
@@ -408,18 +516,12 @@ public:
   String &treewalk(){return treew;};
   String &preoutput(){return preout;};
   String &postoutput(){return postout;};
+
   String prefix();
 };
 
-class menuentry {
-private:
-  char hotkeyconv(char);
-  void generate_hotkeys();
-public:
-  map <String, menuentry *, less<String> > submenus;
-  map <String, String, less<String> > vars;
-  void add_entry(String, String, map <String, String, less<String> > &);
-  void output();
-  void postprocess();
-};
+extern configinfo *config;
+extern supportedinfo *supported;
 
+
+#endif /* MENU_METHOD_H */
